// Import the contracts to test in Truffle
const MyContract = artifacts.require("MyContract");

// Create a testing environment. accounts are Ethereum accounts 
contract("Testing MyContract", accounts => {

    it("Should test the constructor", function() {
        // 1 Create a known state: not here, we test the constructor
        // 2 Execute the operation to test, i.e. the constructor     
        return MyContract.new() // Create a new contract
            .then(instance => {
                // instance is a constructed instance of MyContract
                // 3 Test the condition: i.e. if value is 1
                return instance.value().then(v => {
                    // v is the result of value(): solidity uint are BigNumber objets
                    assert.equal(v.toNumber(), 1, "Value should be initialized at 1");
                });
            });
    });

    it("Should test the constructor with async/await", async function() {
        // 1 Create a known state: not here, we test the constructor
        // 2 Execute the operation to test, i.e. the constructor
        const instance = await MyContract.new();
        // 3 Test the condition: i.e. if value is 1
        const v = await instance.value();
        assert.equal(v.toNumber(), 1, "Value should be initialized at 1");
    });

    it("Should increase the value by 41", async function() {
        // 1 Create a known state
        instance = await MyContract.new();
        // 2 Execute the operation to test
        const result = await instance.increase(41); // it is a tx object
        // 3 Test the condition
        assert.notEqual(result, 42, "The result should not be 42");
        // console.log(result);
        const v = await instance.value();
        assert.equal(v, 42, "The result should be 42");
    });

    it("Should fire event", async function() {
        // 1 Create a known state
        instance = await MyContract.new();
        // 2 Execute the operation to test
        let tx = await instance.press_click();
        console.log(tx);
        console.log(tx.logs[1].args);

    });
    
});
