const fs = require('fs');

// Smart contracts
const contract = require("@truffle/contract");
const Web3 = require("web3");
const contract_data = require("./build/contracts/IPFSContract.json");

// IPFS
const ipfsClient = require("ipfs-http-client") ;
const {globSource} = ipfsClient;

const localhost = "http://127.0.0.1:8545";
const web3 = new Web3(localhost);
const provider = new Web3.providers.HttpProvider(localhost);

const IPFSContract = contract(contract_data);
IPFSContract.setProvider(provider);

async function main() {

    const accounts = await web3.eth.getAccounts();
    const alice = accounts[0];
    
    // Create the contract
    const instance = await IPFSContract.new({from: alice});
    // const instance = await IPFSContract.deployed();
    // const instance = await IPFSContract.at(contract_address);
    console.log("Cid: " + await instance.cid());

    // Connect IPFS
    // Connect to local node
    // const ipfs = ipfsClient("localhost", "5001", {protocol: 'http'});

    // Connect to Infura with public gateway
    const ipfs = ipfsClient({host: "ipfs.infura.io", port: "5001", protocol: 'https'});

    // Read filename from command line
    const args = process.argv.slice(2);
    const file_name = args[0];
    let file_ipfs_data;
    // Store the file on IPFS and store on 
    for await (const f of ipfs.add(globSource(file_name))) {
        file_ipfs_data = f;
        console.log(file_ipfs_data);
    }

    // Store the CID in the smart contract
    const tx = await instance.setCid(file_ipfs_data.cid.toString(), {from: alice});
    const cid = await instance.cid();
    console.log("Cid: " + cid);

    // Retrieve the file
    const bytes = [];
    for await (const chunk of ipfs.cat(cid)) {
        bytes.push(chunk);
    }

    console.log(Buffer.concat(bytes).toString());

    // Terminate process
    process.exit();
}

main();
